      subroutine mmv3 ( mat3, vec, bresul, nxa, nya, nza,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c       version 1.2    date  8- 7-1997
c       version 1.1    10-04-1997
c        version 1.0    date Tue Apr  8 1997
c
c   copyright (c) 1997
c   all rights bresulerved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 2d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c  calculates Ax on any level
c
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nxa, nya, nza
      integer sl1, sl2, sl3, sl4, sl5, sl6
      double precision
     +  mat3 (1-sl1:nxa+sl2, 1-sl3:nya+sl4,
     +  1-sl5:nza+sl6, 27),
     +  vec (1-sl1:nxa+sl2, 1-sl3:nya+sl4,
     +  1-sl5:nza+sl6),
     +  bresul (1-sl1:nxa+sl2, 1-sl3:nya+sl4,
     +  1-sl5:nza+sl6)

c     bresul    i    The result vector
c     mat3      i    The nx x ny x nz x 27  coefficient matrix of the
c                    discrete Laplacian
c     nxa       i    Number of grid x points on this level
c     nya       i    Number of grid y points on this level
c     nza       i    Number of grid z points on this level
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c     vec       i    Generic name for quantity with the dimension of
c                    sol or rhs
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixa, iya, iza

c     flevel0   Flag to denote whether we are on level 0
c               (module mgpreset)
c     ixa       Counter to number of grid x points on this level
c     iya       Counter to number of grid y points on this level
c     iza       Counter to number of grid z points on this level
c     molstruc  To signify the molecule structure on the finest level:
c               pt27 (default), pt19 or pt7 (module mgpreset)
c     p000      Index of point in stencil 3D (module mgpreset)
c     p001      Index of point in stencil 3D (module mgpreset)
c     p002      Index of point in stencil 3D (module mgpreset)
c     p010      Index of point in stencil 3D (module mgpreset)
c     p011      Index of point in stencil 3D (module mgpreset)
c     p012      Index of point in stencil 3D (module mgpreset)
c     p020      Index of point in stencil 3D (module mgpreset)
c     p021      Index of point in stencil 3D (module mgpreset)
c     p022      Index of point in stencil 3D (module mgpreset)
c     p100      Index of point in stencil 3D (module mgpreset)
c     p101      Index of point in stencil 3D (module mgpreset)
c     p102      Index of point in stencil 3D (module mgpreset)
c     p110      Index of point in stencil 3D (module mgpreset)
c     p111      Index of point in stencil 3D (module mgpreset)
c     p112      Index of point in stencil 3D (module mgpreset)
c     p120      Index of point in stencil 3D (module mgpreset)
c     p121      Index of point in stencil 3D (module mgpreset)
c     p122      Index of point in stencil 3D (module mgpreset)
c     p200      Index of point in stencil 3D (module mgpreset)
c     p201      Index of point in stencil 3D (module mgpreset)
c     p202      Index of point in stencil 3D (module mgpreset)
c     p210      Index of point in stencil 3D (module mgpreset)
c     p211      Index of point in stencil 3D (module mgpreset)
c     p212      Index of point in stencil 3D (module mgpreset)
c     p220      Index of point in stencil 3D (module mgpreset)
c     p221      Index of point in stencil 3D (module mgpreset)
c     p222      Index of point in stencil 3D (module mgpreset)
c     pt19      Named constant (1). Molecule has 19 points on level 0
c               (module mgpreset)
c     pt27      Named constant (0). Molecule has 27 points on level 0
c               (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     M19MV3    Matrix vector mult (level 0) 19 pt stencil (MG3)
c     M7MV3     Matrix vector mult (level 0) 7 pt stencil (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c   trivial
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Specially structured matrices on level 0 are treated separately.
c         On higher levels the molecule structure always is 27 pt full.

      if (flevel0 .and. (molstruc.eq.pt19)) then
         call m19mv3 ( mat3, vec, bresul, nxa, nya, nza,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
        return
      end if

      if (flevel0 .and. (molstruc.eq.pt7)) then
        call m7mv3 ( mat3, vec, bresul, nxa, nya, nza,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
        return
      end if

c     --- You could do this in a sloppy way: multiply everything.
c         this would be all right, since all mat3rix coefficients that do not
c         contribute are 0.
c         We prefer to do it the right way. Somewhat more lines though. (27)
c         --- The corners (8)

      bresul (1, 1, 1) =
     +       mat3 (1, 1, 1, p111) * vec (1, 1, 1)+
     +       mat3 (1, 1, 1, p112) * vec (1, 1, 2)+
     +       mat3 (1, 1, 1, p121) * vec (1, 2, 1)+
     +       mat3 (1, 1, 1, p122) * vec (1, 2, 2)+
     +       mat3 (1, 1, 1, p211) * vec (2, 1, 1)+
     +       mat3 (1, 1, 1, p212) * vec (2, 1, 2)+
     +       mat3 (1, 1, 1, p221) * vec (2, 2, 1)+
     +       mat3 (1, 1, 1, p222) * vec (2, 2, 2)
      bresul (nxa, 1, 1) =
     +       mat3 (nxa, 1, 1, p011) * vec (nxa-1, 1, 1)+
     +       mat3 (nxa, 1, 1, p012) * vec (nxa-1, 1, 2)+
     +       mat3 (nxa, 1, 1, p021) * vec (nxa-1, 2, 1)+
     +       mat3 (nxa, 1, 1, p022) * vec (nxa-1, 2, 2)+
     +       mat3 (nxa, 1, 1, p111) * vec (nxa, 1, 1)+
     +       mat3 (nxa, 1, 1, p112) * vec (nxa, 1, 2)+
     +       mat3 (nxa, 1, 1, p121) * vec (nxa, 2, 1)+
     +       mat3 (nxa, 1, 1, p122) * vec (nxa, 2, 2)
      bresul (1, nya, 1) =
     +       mat3 (1, nya, 1, p101) * vec (1, nya-1, 1)+
     +       mat3 (1, nya, 1, p102) * vec (1, nya-1, 2)+
     +       mat3 (1, nya, 1, p111) * vec (1, nya, 1)+
     +       mat3 (1, nya, 1, p112) * vec (1, nya, 2)+
     +       mat3 (1, nya, 1, p201) * vec (2, nya-1, 1)+
     +       mat3 (1, nya, 1, p202) * vec (2, nya-1, 2)+
     +       mat3 (1, nya, 1, p211) * vec (2, nya, 1)+
     +       mat3 (1, nya, 1, p212) * vec (2, nya, 2)
      bresul (nxa, nya, 1) =
     +       mat3 (nxa, nya, 1, p001) * vec (nxa-1, nya-1, 1)+
     +       mat3 (nxa, nya, 1, p002) * vec (nxa-1, nya-1, 2)+
     +       mat3 (nxa, nya, 1, p011) * vec (nxa-1, nya, 1)+
     +       mat3 (nxa, nya, 1, p012) * vec (nxa-1, nya, 2)+
     +       mat3 (nxa, nya, 1, p101) * vec (nxa, nya-1, 1)+
     +       mat3 (nxa, nya, 1, p102) * vec (nxa, nya-1, 2)+
     +       mat3 (nxa, nya, 1, p111) * vec (nxa, nya, 1)+
     +       mat3 (nxa, nya, 1, p112) * vec (nxa, nya, 2)
      bresul (1, 1, nza) =
     +       mat3 (1, 1, nza, p110) * vec (1, 1, nza-1)+
     +       mat3 (1, 1, nza, p111) * vec (1, 1, nza)+
     +       mat3 (1, 1, nza, p120) * vec (1, 2, nza-1)+
     +       mat3 (1, 1, nza, p121) * vec (1, 2, nza)+
     +       mat3 (1, 1, nza, p210) * vec (2, 1, nza-1)+
     +       mat3 (1, 1, nza, p211) * vec (2, 1, nza)+
     +       mat3 (1, 1, nza, p220) * vec (2, 2, nza-1)+
     +       mat3 (1, 1, nza, p221) * vec (2, 1+1, nza)
      bresul (nxa, 1, nza) =
     +       mat3 (nxa, 1, nza, p010) * vec (nxa-1, 1, nza-1)+
     +       mat3 (nxa, 1, nza, p011) * vec (nxa-1, 1, nza)+
     +       mat3 (nxa, 1, nza, p020) * vec (nxa-1, 2, nza-1)+
     +       mat3 (nxa, 1, nza, p021) * vec (nxa-1, 2, nza)+
     +       mat3 (nxa, 1, nza, p110) * vec (nxa, 1, nza-1)+
     +       mat3 (nxa, 1, nza, p111) * vec (nxa, 1, nza)+
     +       mat3 (nxa, 1, nza, p120) * vec (nxa, 2, nza-1)+
     +       mat3 (nxa, 1, nza, p121) * vec (nxa, 2, nza)
      bresul (1, nya, nza) =
     +       mat3 (1, nya, nza, p100) * vec (1, nya-1, nza-1)+
     +       mat3 (1, nya, nza, p101) * vec (1, nya-1, nza)+
     +       mat3 (1, nya, nza, p110) * vec (1, nya, nza-1)+
     +       mat3 (1, nya, nza, p111) * vec (1, nya, nza)+
     +       mat3 (1, nya, nza, p200) * vec (2, nya-1, nza-1)+
     +       mat3 (1, nya, nza, p201) * vec (2, nya-1, nza)+
     +       mat3 (1, nya, nza, p210) * vec (2, nya, nza-1)+
     +       mat3 (1, nya, nza, p211) * vec (2, nya, nza)
      bresul (nxa, nya, nza) =
     +       mat3 (nxa, nya, nza, p000) * vec (nxa-1, nya-1, nza-1)+
     +       mat3 (nxa, nya, nza, p001) * vec (nxa-1, nya-1, nza)+
     +       mat3 (nxa, nya, nza, p010) * vec (nxa-1, nya, nza-1)+
     +       mat3 (nxa, nya, nza, p011) * vec (nxa-1, nya, nza)+
     +       mat3 (nxa, nya, nza, p100) * vec (nxa, nya-1, nza-1)+
     +       mat3 (nxa, nya, nza, p101) * vec (nxa, nya-1, nza)+
     +       mat3 (nxa, nya, nza, p110) * vec (nxa, nya, nza-1)+
     +       mat3 (nxa, nya, nza, p111) * vec (nxa, nya, nza)

c     --- The edges (12)

      do ixa = 2, nxa - 1
      bresul (ixa, 1, 1) =
     +       mat3 (ixa, 1, 1, p011) * vec (ixa-1, 1, 1)+
     +       mat3 (ixa, 1, 1, p012) * vec (ixa-1, 1, 2)+
     +       mat3 (ixa, 1, 1, p021) * vec (ixa-1, 2, 1)+
     +       mat3 (ixa, 1, 1, p022) * vec (ixa-1, 2, 2)+
     +       mat3 (ixa, 1, 1, p111) * vec (ixa, 1, 1)+
     +       mat3 (ixa, 1, 1, p112) * vec (ixa, 1, 2)+
     +       mat3 (ixa, 1, 1, p121) * vec (ixa, 2, 1)+
     +       mat3 (ixa, 1, 1, p122) * vec (ixa, 2, 2)+
     +       mat3 (ixa, 1, 1, p211) * vec (ixa+1, 1, 1)+
     +       mat3 (ixa, 1, 1, p212) * vec (ixa+1, 1, 2)+
     +       mat3 (ixa, 1, 1, p221) * vec (ixa+1, 2, 1)+
     +       mat3 (ixa, 1, 1, p222) * vec (ixa+1, 2, 2)
      bresul (ixa, nya, 1) =
     +       mat3 (ixa, nya, 1, p001) * vec (ixa-1, nya-1, 1)+
     +       mat3 (ixa, nya, 1, p002) * vec (ixa-1, nya-1, 2)+
     +       mat3 (ixa, nya, 1, p011) * vec (ixa-1, nya, 1)+
     +       mat3 (ixa, nya, 1, p012) * vec (ixa-1, nya, 2)+
     +       mat3 (ixa, nya, 1, p101) * vec (ixa, nya-1, 1)+
     +       mat3 (ixa, nya, 1, p102) * vec (ixa, nya-1, 2)+
     +       mat3 (ixa, nya, 1, p111) * vec (ixa, nya, 1)+
     +       mat3 (ixa, nya, 1, p112) * vec (ixa, nya, 2)+
     +       mat3 (ixa, nya, 1, p201) * vec (ixa+1, nya-1, 1)+
     +       mat3 (ixa, nya, 1, p202) * vec (ixa+1, nya-1, 2)+
     +       mat3 (ixa, nya, 1, p211) * vec (ixa+1, nya, 1)+
     +       mat3 (ixa, nya, 1, p212) * vec (ixa+1, nya, 2)
      bresul (ixa, 1, nza) =
     +       mat3 (ixa, 1, nza, p010) * vec (ixa-1, 1, nza-1)+
     +       mat3 (ixa, 1, nza, p011) * vec (ixa-1, 1, nza)+
     +       mat3 (ixa, 1, nza, p020) * vec (ixa-1, 2, nza-1)+
     +       mat3 (ixa, 1, nza, p021) * vec (ixa-1, 2, nza)+
     +       mat3 (ixa, 1, nza, p110) * vec (ixa, 1, nza-1)+
     +       mat3 (ixa, 1, nza, p111) * vec (ixa, 1, nza)+
     +       mat3 (ixa, 1, nza, p120) * vec (ixa, 2, nza-1)+
     +       mat3 (ixa, 1, nza, p121) * vec (ixa, 2, nza)+
     +       mat3 (ixa, 1, nza, p210) * vec (ixa+1, 1, nza-1)+
     +       mat3 (ixa, 1, nza, p211) * vec (ixa+1, 1, nza)+
     +       mat3 (ixa, 1, nza, p220) * vec (ixa+1, 2, nza-1)+
     +       mat3 (ixa, 1, nza, p221) * vec (ixa+1, 2, nza)
      bresul (ixa, nya, nza) =
     +       mat3 (ixa, nya, nza, p000) * vec (ixa-1, nya-1, nza-1)+
     +       mat3 (ixa, nya, nza, p001) * vec (ixa-1, nya-1, nza)+
     +       mat3 (ixa, nya, nza, p010) * vec (ixa-1, nya, nza-1)+
     +       mat3 (ixa, nya, nza, p011) * vec (ixa-1, nya, nza)+
     +       mat3 (ixa, nya, nza, p100) * vec (ixa, nya-1, nza-1)+
     +       mat3 (ixa, nya, nza, p101) * vec (ixa, nya-1, nza)+
     +       mat3 (ixa, nya, nza, p110) * vec (ixa, nya, nza-1)+
     +       mat3 (ixa, nya, nza, p111) * vec (ixa, nya, nza)+
     +       mat3 (ixa, nya, nza, p200) * vec (ixa+1, nya-1, nza-1)+
     +       mat3 (ixa, nya, nza, p201) * vec (ixa+1, nya-1, nza)+
     +       mat3 (ixa, nya, nza, p210) * vec (ixa+1, nya, nza-1)+
     +       mat3 (ixa, nya, nza, p211) * vec (ixa+1, nya, nza)
      end do

      do iya = 2, nya - 1
      bresul (1, iya, 1) =
     +       mat3 (1, iya, 1, p101) * vec (1, iya-1, 1)+
     +       mat3 (1, iya, 1, p102) * vec (1, iya-1, 2)+
     +       mat3 (1, iya, 1, p111) * vec (1, iya, 1)+
     +       mat3 (1, iya, 1, p112) * vec (1, iya, 2)+
     +       mat3 (1, iya, 1, p121) * vec (1, iya+1, 1)+
     +       mat3 (1, iya, 1, p122) * vec (1, iya+1, 2)+
     +       mat3 (1, iya, 1, p201) * vec (1+1, iya-1, 1)+
     +       mat3 (1, iya, 1, p202) * vec (2, iya-1, 2)+
     +       mat3 (1, iya, 1, p211) * vec (2, iya, 1)+
     +       mat3 (1, iya, 1, p212) * vec (2, iya, 2)+
     +       mat3 (1, iya, 1, p221) * vec (2, iya+1, 1)+
     +       mat3 (1, iya, 1, p222) * vec (2, iya+1, 2)
      bresul (nxa, iya, 1) =
     +       mat3 (nxa, iya, 1, p001) * vec (nxa-1, iya-1, 1)+
     +       mat3 (nxa, iya, 1, p002) * vec (nxa-1, iya-1, 2)+
     +       mat3 (nxa, iya, 1, p011) * vec (nxa-1, iya, 1)+
     +       mat3 (nxa, iya, 1, p012) * vec (nxa-1, iya, 2)+
     +       mat3 (nxa, iya, 1, p021) * vec (nxa-1, iya+1, 1)+
     +       mat3 (nxa, iya, 1, p022) * vec (nxa-1, iya+1, 2)+
     +       mat3 (nxa, iya, 1, p101) * vec (nxa, iya-1, 1)+
     +       mat3 (nxa, iya, 1, p102) * vec (nxa, iya-1, 2)+
     +       mat3 (nxa, iya, 1, p111) * vec (nxa, iya, 1)+
     +       mat3 (nxa, iya, 1, p112) * vec (nxa, iya, 2)+
     +       mat3 (nxa, iya, 1, p121) * vec (nxa, iya+1, 1)+
     +       mat3 (nxa, iya, 1, p122) * vec (nxa, iya+1, 2)
      bresul (1, iya, nza) =
     +       mat3 (1, iya, nza, p100) * vec (1, iya-1, nza-1)+
     +       mat3 (1, iya, nza, p101) * vec (1, iya-1, nza)+
     +       mat3 (1, iya, nza, p110) * vec (1, iya, nza-1)+
     +       mat3 (1, iya, nza, p111) * vec (1, iya, nza)+
     +       mat3 (1, iya, nza, p120) * vec (1, iya+1, nza-1)+
     +       mat3 (1, iya, nza, p121) * vec (1, iya+1, nza)+
     +       mat3 (1, iya, nza, p200) * vec (1+1, iya-1, nza-1)+
     +       mat3 (1, iya, nza, p201) * vec (1+1, iya-1, nza)+
     +       mat3 (1, iya, nza, p210) * vec (1+1, iya, nza-1)+
     +       mat3 (1, iya, nza, p211) * vec (1+1, iya, nza)+
     +       mat3 (1, iya, nza, p220) * vec (1+1, iya+1, nza-1)+
     +       mat3 (1, iya, nza, p221) * vec (1+1, iya+1, nza)
      bresul (nxa, iya, nza) =
     +       mat3 (nxa, iya, nza, p000) * vec (nxa-1, iya-1, nza-1)+
     +       mat3 (nxa, iya, nza, p001) * vec (nxa-1, iya-1, nza)+
     +       mat3 (nxa, iya, nza, p010) * vec (nxa-1, iya, nza-1)+
     +       mat3 (nxa, iya, nza, p011) * vec (nxa-1, iya, nza)+
     +       mat3 (nxa, iya, nza, p020) * vec (nxa-1, iya+1, nza-1)+
     +       mat3 (nxa, iya, nza, p021) * vec (nxa-1, iya+1, nza)+
     +       mat3 (nxa, iya, nza, p100) * vec (nxa, iya-1, nza-1)+
     +       mat3 (nxa, iya, nza, p101) * vec (nxa, iya-1, nza)+
     +       mat3 (nxa, iya, nza, p110) * vec (nxa, iya, nza-1)+
     +       mat3 (nxa, iya, nza, p111) * vec (nxa, iya, nza)+
     +       mat3 (nxa, iya, nza, p120) * vec (nxa, iya+1, nza-1)+
     +       mat3 (nxa, iya, nza, p121) * vec (nxa, iya+1, nza)
      end do

      do iza = 2, nza - 1
      bresul (1, 1, iza) =
     +       mat3 (1, 1, iza, p110) * vec (1, 1, iza-1)+
     +       mat3 (1, 1, iza, p111) * vec (1, 1, iza)+
     +       mat3 (1, 1, iza, p112) * vec (1, 1, iza+1)+
     +       mat3 (1, 1, iza, p120) * vec (1, 2, iza-1)+
     +       mat3 (1, 1, iza, p121) * vec (1, 2, iza)+
     +       mat3 (1, 1, iza, p122) * vec (1, 2, iza+1)+
     +       mat3 (1, 1, iza, p210) * vec (2, 1, iza-1)+
     +       mat3 (1, 1, iza, p211) * vec (2, 1, iza)+
     +       mat3 (1, 1, iza, p212) * vec (2, 1, iza+1)+
     +       mat3 (1, 1, iza, p220) * vec (2, 2, iza-1)+
     +       mat3 (1, 1, iza, p221) * vec (2, 2, iza)+
     +       mat3 (1, 1, iza, p222) * vec (2, 2, iza+1)
      bresul (nxa, 1, iza) =
     +       mat3 (nxa, 1, iza, p010) * vec (nxa-1, 1, iza-1)+
     +       mat3 (nxa, 1, iza, p011) * vec (nxa-1, 1, iza)+
     +       mat3 (nxa, 1, iza, p012) * vec (nxa-1, 1, iza+1)+
     +       mat3 (nxa, 1, iza, p020) * vec (nxa-1, 2, iza-1)+
     +       mat3 (nxa, 1, iza, p021) * vec (nxa-1, 2, iza)+
     +       mat3 (nxa, 1, iza, p022) * vec (nxa-1, 2, iza+1)+
     +       mat3 (nxa, 1, iza, p110) * vec (nxa, 1, iza-1)+
     +       mat3 (nxa, 1, iza, p111) * vec (nxa, 1, iza)+
     +       mat3 (nxa, 1, iza, p112) * vec (nxa, 1, iza+1)+
     +       mat3 (nxa, 1, iza, p120) * vec (nxa, 2, iza-1)+
     +       mat3 (nxa, 1, iza, p121) * vec (nxa, 2, iza)+
     +       mat3 (nxa, 1, iza, p122) * vec (nxa, 2, iza+1)
      bresul (1, nya, iza) =
     +       mat3 (1, nya, iza, p100) * vec (1, nya-1, iza-1)+
     +       mat3 (1, nya, iza, p101) * vec (1, nya-1, iza)+
     +       mat3 (1, nya, iza, p102) * vec (1, nya-1, iza+1)+
     +       mat3 (1, nya, iza, p110) * vec (1, nya, iza-1)+
     +       mat3 (1, nya, iza, p111) * vec (1, nya, iza)+
     +       mat3 (1, nya, iza, p112) * vec (1, nya, iza+1)+
     +       mat3 (1, nya, iza, p200) * vec (2, nya-1, iza-1)+
     +       mat3 (1, nya, iza, p201) * vec (2, nya-1, iza)+
     +       mat3 (1, nya, iza, p202) * vec (2, nya-1, iza+1)+
     +       mat3 (1, nya, iza, p210) * vec (2, nya, iza-1)+
     +       mat3 (1, nya, iza, p211) * vec (2, nya, iza)+
     +       mat3 (1, nya, iza, p212) * vec (2, nya, iza+1)
      bresul (nxa, nya, iza) =
     +       mat3 (nxa, nya, iza, p000) * vec (nxa-1, nya-1, iza-1)+
     +       mat3 (nxa, nya, iza, p001) * vec (nxa-1, nya-1, iza)+
     +       mat3 (nxa, nya, iza, p002) * vec (nxa-1, nya-1, iza+1)+
     +       mat3 (nxa, nya, iza, p010) * vec (nxa-1, nya, iza-1)+
     +       mat3 (nxa, nya, iza, p011) * vec (nxa-1, nya, iza)+
     +       mat3 (nxa, nya, iza, p012) * vec (nxa-1, nya, iza+1)+
     +       mat3 (nxa, nya, iza, p100) * vec (nxa, nya-1, iza-1)+
     +       mat3 (nxa, nya, iza, p101) * vec (nxa, nya-1, iza)+
     +       mat3 (nxa, nya, iza, p102) * vec (nxa, nya-1, iza+1)+
     +       mat3 (nxa, nya, iza, p110) * vec (nxa, nya, iza-1)+
     +       mat3 (nxa, nya, iza, p111) * vec (nxa, nya, iza)+
     +       mat3 (nxa, nya, iza, p112) * vec (nxa, nya, iza+1)
      end do

c     --- the faces (6)

      do iya = 2, nya - 1
      do ixa = 2, nxa - 1
      bresul (ixa, iya, 1) =
     +       mat3 (ixa, iya, 1, p001) * vec (ixa-1, iya-1, 1)+
     +       mat3 (ixa, iya, 1, p002) * vec (ixa-1, iya-1, 2)+
     +       mat3 (ixa, iya, 1, p011) * vec (ixa-1, iya, 1)+
     +       mat3 (ixa, iya, 1, p012) * vec (ixa-1, iya, 2)+
     +       mat3 (ixa, iya, 1, p021) * vec (ixa-1, iya+1, 1)+
     +       mat3 (ixa, iya, 1, p022) * vec (ixa-1, iya+1, 2)+
     +       mat3 (ixa, iya, 1, p101) * vec (ixa, iya-1, 1)+
     +       mat3 (ixa, iya, 1, p102) * vec (ixa, iya-1, 2)+
     +       mat3 (ixa, iya, 1, p111) * vec (ixa, iya, 1)+
     +       mat3 (ixa, iya, 1, p112) * vec (ixa, iya, 2)+
     +       mat3 (ixa, iya, 1, p121) * vec (ixa, iya+1, 1)+
     +       mat3 (ixa, iya, 1, p122) * vec (ixa, iya+1, 2)+
     +       mat3 (ixa, iya, 1, p201) * vec (ixa+1, iya-1, 1)+
     +       mat3 (ixa, iya, 1, p202) * vec (ixa+1, iya-1, 2)+
     +       mat3 (ixa, iya, 1, p211) * vec (ixa+1, iya, 1)+
     +       mat3 (ixa, iya, 1, p212) * vec (ixa+1, iya, 2)+
     +       mat3 (ixa, iya, 1, p221) * vec (ixa+1, iya+1, 1)+
     +       mat3 (ixa, iya, 1, p222) * vec (ixa+1, iya+1, 2)
      bresul (ixa, iya, nza) =
     +       mat3 (ixa, iya, nza, p000) * vec (ixa-1, iya-1, nza-1)+
     +       mat3 (ixa, iya, nza, p001) * vec (ixa-1, iya-1, nza)+
     +       mat3 (ixa, iya, nza, p010) * vec (ixa-1, iya, nza-1)+
     +       mat3 (ixa, iya, nza, p011) * vec (ixa-1, iya, nza)+
     +       mat3 (ixa, iya, nza, p020) * vec (ixa-1, iya+1, nza-1)+
     +       mat3 (ixa, iya, nza, p021) * vec (ixa-1, iya+1, nza)+
     +       mat3 (ixa, iya, nza, p100) * vec (ixa, iya-1, nza-1)+
     +       mat3 (ixa, iya, nza, p101) * vec (ixa, iya-1, nza)+
     +       mat3 (ixa, iya, nza, p110) * vec (ixa, iya, nza-1)+
     +       mat3 (ixa, iya, nza, p111) * vec (ixa, iya, nza)+
     +       mat3 (ixa, iya, nza, p120) * vec (ixa, iya+1, nza-1)+
     +       mat3 (ixa, iya, nza, p121) * vec (ixa, iya+1, nza)+
     +       mat3 (ixa, iya, nza, p200) * vec (ixa+1, iya-1, nza-1)+
     +       mat3 (ixa, iya, nza, p201) * vec (ixa+1, iya-1, nza)+
     +       mat3 (ixa, iya, nza, p210) * vec (ixa+1, iya, nza-1)+
     +       mat3 (ixa, iya, nza, p211) * vec (ixa+1, iya, nza)+
     +       mat3 (ixa, iya, nza, p220) * vec (ixa+1, iya+1, nza-1)+
     +       mat3 (ixa, iya, nza, p221) * vec (ixa+1, iya+1, nza)
      end do
      end do

      do iza = 2, nza - 1
      do ixa = 2, nxa - 1
      bresul (ixa, 1, iza) =
     +       mat3 (ixa, 1, iza, p010) * vec (ixa-1, 1, iza-1)+
     +       mat3 (ixa, 1, iza, p011) * vec (ixa-1, 1, iza)+
     +       mat3 (ixa, 1, iza, p012) * vec (ixa-1, 1, iza+1)+
     +       mat3 (ixa, 1, iza, p020) * vec (ixa-1, 2, iza-1)+
     +       mat3 (ixa, 1, iza, p021) * vec (ixa-1, 2, iza)+
     +       mat3 (ixa, 1, iza, p022) * vec (ixa-1, 2, iza+1)+
     +       mat3 (ixa, 1, iza, p110) * vec (ixa, 1, iza-1)+
     +       mat3 (ixa, 1, iza, p111) * vec (ixa, 1, iza)+
     +       mat3 (ixa, 1, iza, p112) * vec (ixa, 1, iza+1)+
     +       mat3 (ixa, 1, iza, p120) * vec (ixa, 2, iza-1)+
     +       mat3 (ixa, 1, iza, p121) * vec (ixa, 2, iza)+
     +       mat3 (ixa, 1, iza, p122) * vec (ixa, 2, iza+1)+
     +       mat3 (ixa, 1, iza, p210) * vec (ixa+1, 1, iza-1)+
     +       mat3 (ixa, 1, iza, p211) * vec (ixa+1, 1, iza)+
     +       mat3 (ixa, 1, iza, p212) * vec (ixa+1, 1, iza+1)+
     +       mat3 (ixa, 1, iza, p220) * vec (ixa+1, 2, iza-1)+
     +       mat3 (ixa, 1, iza, p221) * vec (ixa+1, 2, iza)+
     +       mat3 (ixa, 1, iza, p222) * vec (ixa+1, 2, iza+1)
      bresul (ixa, nya, iza) =
     +       mat3 (ixa, nya, iza, p000) * vec (ixa-1, nya-1, iza-1)+
     +       mat3 (ixa, nya, iza, p001) * vec (ixa-1, nya-1, iza)+
     +       mat3 (ixa, nya, iza, p002) * vec (ixa-1, nya-1, iza+1)+
     +       mat3 (ixa, nya, iza, p010) * vec (ixa-1, nya, iza-1)+
     +       mat3 (ixa, nya, iza, p011) * vec (ixa-1, nya, iza)+
     +       mat3 (ixa, nya, iza, p012) * vec (ixa-1, nya, iza+1)+
     +       mat3 (ixa, nya, iza, p100) * vec (ixa, nya-1, iza-1)+
     +       mat3 (ixa, nya, iza, p101) * vec (ixa, nya-1, iza)+
     +       mat3 (ixa, nya, iza, p102) * vec (ixa, nya-1, iza+1)+
     +       mat3 (ixa, nya, iza, p110) * vec (ixa, nya, iza-1)+
     +       mat3 (ixa, nya, iza, p111) * vec (ixa, nya, iza)+
     +       mat3 (ixa, nya, iza, p112) * vec (ixa, nya, iza+1)+
     +       mat3 (ixa, nya, iza, p200) * vec (ixa+1, nya-1, iza-1)+
     +       mat3 (ixa, nya, iza, p201) * vec (ixa+1, nya-1, iza)+
     +       mat3 (ixa, nya, iza, p202) * vec (ixa+1, nya-1, iza+1)+
     +       mat3 (ixa, nya, iza, p210) * vec (ixa+1, nya, iza-1)+
     +       mat3 (ixa, nya, iza, p211) * vec (ixa+1, nya, iza)+
     +       mat3 (ixa, nya, iza, p212) * vec (ixa+1, nya, iza+1)
      end do
      end do

      do iza = 2, nza - 1
      do iya = 2, nya - 1
      bresul (1, iya, iza) =
     +       mat3 (1, iya, iza, p100) * vec (1, iya-1, iza-1)+
     +       mat3 (1, iya, iza, p101) * vec (1, iya-1, iza)+
     +       mat3 (1, iya, iza, p102) * vec (1, iya-1, iza+1)+
     +       mat3 (1, iya, iza, p110) * vec (1, iya, iza-1)+
     +       mat3 (1, iya, iza, p111) * vec (1, iya, iza)+
     +       mat3 (1, iya, iza, p112) * vec (1, iya, iza+1)+
     +       mat3 (1, iya, iza, p120) * vec (1, iya+1, iza-1)+
     +       mat3 (1, iya, iza, p121) * vec (1, iya+1, iza)+
     +       mat3 (1, iya, iza, p122) * vec (1, iya+1, iza+1)+
     +       mat3 (1, iya, iza, p200) * vec (2, iya-1, iza-1)+
     +       mat3 (1, iya, iza, p201) * vec (2, iya-1, iza)+
     +       mat3 (1, iya, iza, p202) * vec (2, iya-1, iza+1)+
     +       mat3 (1, iya, iza, p210) * vec (2, iya, iza-1)+
     +       mat3 (1, iya, iza, p211) * vec (2, iya, iza)+
     +       mat3 (1, iya, iza, p212) * vec (2, iya, iza+1)+
     +       mat3 (1, iya, iza, p220) * vec (2, iya+1, iza-1)+
     +       mat3 (1, iya, iza, p221) * vec (2, iya+1, iza)+
     +       mat3 (1, iya, iza, p222) * vec (2, iya+1, iza+1)
      bresul (nxa, iya, iza) =
     +       mat3 (nxa, iya, iza, p000) * vec (nxa-1, iya-1, iza-1)+
     +       mat3 (nxa, iya, iza, p001) * vec (nxa-1, iya-1, iza)+
     +       mat3 (nxa, iya, iza, p002) * vec (nxa-1, iya-1, iza+1)+
     +       mat3 (nxa, iya, iza, p010) * vec (nxa-1, iya, iza-1)+
     +       mat3 (nxa, iya, iza, p011) * vec (nxa-1, iya, iza)+
     +       mat3 (nxa, iya, iza, p012) * vec (nxa-1, iya, iza+1)+
     +       mat3 (nxa, iya, iza, p020) * vec (nxa-1, iya+1, iza-1)+
     +       mat3 (nxa, iya, iza, p021) * vec (nxa-1, iya+1, iza)+
     +       mat3 (nxa, iya, iza, p022) * vec (nxa-1, iya+1, iza+1)+
     +       mat3 (nxa, iya, iza, p100) * vec (nxa, iya-1, iza-1)+
     +       mat3 (nxa, iya, iza, p101) * vec (nxa, iya-1, iza)+
     +       mat3 (nxa, iya, iza, p102) * vec (nxa, iya-1, iza+1)+
     +       mat3 (nxa, iya, iza, p110) * vec (nxa, iya, iza-1)+
     +       mat3 (nxa, iya, iza, p111) * vec (nxa, iya, iza)+
     +       mat3 (nxa, iya, iza, p112) * vec (nxa, iya, iza+1)+
     +       mat3 (nxa, iya, iza, p120) * vec (nxa, iya+1, iza-1)+
     +       mat3 (nxa, iya, iza, p121) * vec (nxa, iya+1, iza)+
     +       mat3 (nxa, iya, iza, p122) * vec (nxa, iya+1, iza+1)
      end do
      end do

c     --- The interior (generic case)

      do iza = 2, nza - 1
      do iya = 2, nya - 1
      do ixa = 2, nxa - 1
      bresul (ixa, iya, iza) =
     +       mat3 (ixa, iya, iza, p000) * vec (ixa-1, iya-1, iza-1)+
     +       mat3 (ixa, iya, iza, p001) * vec (ixa-1, iya-1, iza)+
     +       mat3 (ixa, iya, iza, p002) * vec (ixa-1, iya-1, iza+1)+
     +       mat3 (ixa, iya, iza, p010) * vec (ixa-1, iya, iza-1)+
     +       mat3 (ixa, iya, iza, p011) * vec (ixa-1, iya, iza)+
     +       mat3 (ixa, iya, iza, p012) * vec (ixa-1, iya, iza+1)+
     +       mat3 (ixa, iya, iza, p020) * vec (ixa-1, iya+1, iza-1)+
     +       mat3 (ixa, iya, iza, p021) * vec (ixa-1, iya+1, iza)+
     +       mat3 (ixa, iya, iza, p022) * vec (ixa-1, iya+1, iza+1)+
     +       mat3 (ixa, iya, iza, p100) * vec (ixa, iya-1, iza-1)+
     +       mat3 (ixa, iya, iza, p101) * vec (ixa, iya-1, iza)+
     +       mat3 (ixa, iya, iza, p102) * vec (ixa, iya-1, iza+1)+
     +       mat3 (ixa, iya, iza, p110) * vec (ixa, iya, iza-1)+
     +       mat3 (ixa, iya, iza, p111) * vec (ixa, iya, iza)+
     +       mat3 (ixa, iya, iza, p112) * vec (ixa, iya, iza+1)+
     +       mat3 (ixa, iya, iza, p120) * vec (ixa, iya+1, iza-1)+
     +       mat3 (ixa, iya, iza, p121) * vec (ixa, iya+1, iza)+
     +       mat3 (ixa, iya, iza, p122) * vec (ixa, iya+1, iza+1)+
     +       mat3 (ixa, iya, iza, p200) * vec (ixa+1, iya-1, iza-1)+
     +       mat3 (ixa, iya, iza, p201) * vec (ixa+1, iya-1, iza)+
     +       mat3 (ixa, iya, iza, p202) * vec (ixa+1, iya-1, iza+1)+
     +       mat3 (ixa, iya, iza, p210) * vec (ixa+1, iya, iza-1)+
     +       mat3 (ixa, iya, iza, p211) * vec (ixa+1, iya, iza)+
     +       mat3 (ixa, iya, iza, p212) * vec (ixa+1, iya, iza+1)+
     +       mat3 (ixa, iya, iza, p220) * vec (ixa+1, iya+1, iza-1)+
     +       mat3 (ixa, iya, iza, p221) * vec (ixa+1, iya+1, iza)+
     +       mat3 (ixa, iya, iza, p222) * vec (ixa+1, iya+1, iza+1)
      end do
      end do
      end do
      end
