      subroutine prolonx3 ( finevec, coarsvec, nxf, nyf, nzf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c       version 1.2    date  8- 7-1997
c       version 1.1    10-04-1997
c        version 1.0    date Tue Apr  8 1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c  prolongation by standard methods in x direction
c  centring method is decided by parity of nfx
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer nxf, nyf, nzf
      integer sl1, sl2, sl3, sl4, sl5, sl6
      double precision
     + finevec (1-sl1:nxf+sl2, 1-sl3:nyf+sl4,
     + 1-sl5:nzf+sl6),
     + coarsvec (nxf/2+1, nyf, nzf)

c     coarsvec  i    Vector on coarse level
c     finevec   o    Vector on fine level
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     nzf       i    Number of grid z points on fine grid
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, izf, ixc, nxc
      integer centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     cellc     Cell centered (module mgpreset)
c     centring  0 = cellcentering, 1 = vertex centering
c     ixc       Counter to number of grid x points on coarse grid
c     ixf       Counter to number of grid x points on fine grid
c     iyf       Counter to number of grid y points on fine grid
c     izf       Counter to number of grid z points on fine grid
c     nxc       Number of grid x points on coarse grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      centring = mod (nxf, 2)
      nxc = nxf/2+1
      aa0 = 0.25d0
      aa1 = 0.75d0
      if (centring.eq.CELLC) then

c     --- cell centered

      do izf = 1, nzf
        do iyf = 1, nyf
          do ixc = 1, nxc-1
           ixf = 2 * ixc-1
           finevec (ixf, iyf, izf) = aa1 * coarsvec (ixc, iyf, izf) +
     +                           aa0 * coarsvec (ixc + 1, iyf, izf)
           finevec (ixf+1, iyf, izf) = aa0 * coarsvec (ixc, iyf, izf) +
     +                          aa1 * coarsvec (ixc + 1, iyf, izf)
          end do
        end do
      end do

c     --- end cell centered

      else

c     --- vertex centered

        aa0 = 0.5d0
        aa1 = 0.5d0
      do izf = 1, nzf
        do iyf = 1, nyf
          finevec (1, iyf, izf) = coarsvec (1, iyf, izf)
          do ixc = 2, nxc
            ixf = 2*ixc - 1
            finevec (ixf - 1,iyf,izf) = aa0*(coarsvec (ixc - 1,iyf,izf)
     +                + coarsvec (ixc, iyf, izf))
            finevec (ixf, iyf, izf) = coarsvec (ixc, iyf, izf)
          end do
        end do
      end do

c     --- end vertex centered

      end if
      end
