      subroutine restri3 (finevec, coarsvec, nxf, nyf, nzf, nxc, nyc,
     +        nzc, rbuf, ibuf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c       version 1.2    date  8- 7-1997
c       version 1.1    10-04-1997
c        version 1.0    date Tue Apr  8 1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   restrict restricts the vector according to the
c   the centring given by the parity of the fine grid.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer ibuf (*)
      double precision finevec (*), coarsvec (*), rbuf(*)
      integer  nxf, nyf, nzf, nxc, nyc, nzc

c     coarsvec  o    Vector on coarse level
c     finevec   i    Vector on fine level
c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     nxc       i    Number of grid x points on coarse grid
c     nxf       i    Number of grid x points on fine grid
c     nyc       i    Number of grid y points on coarse grid
c     nyf       i    Number of grid y points on fine grid
c     nzc       i    Number of grid z points on coarse grid
c     nzf       i    Number of grid z points on fine grid
c     rbuf      i    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer xcoarse, xycoarse

c     offscri   Ibuf (offscri) contains index where scratch space begins
c               in rbuf (module mgpreset)
c     xcoarse   offset in scratch buffer to put intermediate
c               restriction/prolongation
c     xycoarse  offset in scratch buffer to put intermediate
c               restriction/prolongation
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     FREESCS3  Gives reserved scratch space back to buffer (MG3)
c     GETSCS3   Reserves a block of scratch space from buffer (MG3)
c     RESTRX3   Restriction (semicoarsening) in x-direction (MG3)
c     RESTRY3   Restriction (semicoarsening) in y-direction (MG3)
c     RESTRZ3   Restriction (semicoarsening) in z-direction (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if ((nxf.eq.nxc) .and. (nzf.eq.nzc)) then

c     --- semicoarsening y-direction

        call restry3 (finevec, coarsvec, nxf, nyf, nzf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
      else if ((nyf.eq.nyc) .and. (nzf.eq.nzc)) then

c     --- semicoarsening x- direction

        call restrx3 (finevec, coarsvec, nxf, nyf, nzf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
      else if ((nyf.eq.nyc) .and. (nxf.eq.nxc)) then

c     --- semicoarsening z- direction

        call restrz3 (finevec, coarsvec, nxf, nyf, nzf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
      else

c     --- full coarsening
c         --- get some scratch space

      call getscs3 ( ibuf, nzf*nyf * nxc)
      xcoarse = ibuf (offscri)
      call getscs3 ( ibuf, nzf*nyc * nxc)
      xycoarse = ibuf (offscri)

c     --- now restrict first in x then in y direction. basically the order
c          is unimportant, but this is the way we implemented it

      call restrx3 ( finevec, rbuf (xcoarse), nxf, nyf, nzf,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
      call restry3 ( rbuf (xcoarse), rbuf (xycoarse), nxc, nyf, nzf,
     +               0, 0, 0, 0, 0, 0)
      call restrz3 ( rbuf (xycoarse), coarsvec, nxc, nyc, nzf,
     +              0, 0, 0, 0, 0, 0)

c     --- free scratch space (twice)

      call freescs3 (ibuf)
      call freescs3 (ibuf)
      end if
      end
